/**********************************************************************
**
** Copyright (C) 2000 Trolltech AS.  All rights reserved.
**
** This file is part of Qt Designer.
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.trolltech.com/gpl/ for GPL licensing information.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
**********************************************************************/

#include "languageinterfaceimpl.h"
#include <qobject.h>
#include <designerinterface.h>
#include <qfile.h>
#include "yyreg.h"
#include <qmetaobject.h>

LanguageInterfaceImpl::LanguageInterfaceImpl( QUnknownInterface *outer )
    : parent( outer ), ref( 0 )
{
}

ulong LanguageInterfaceImpl::addRef()
{
    return parent ? parent->addRef() : ref++;
}

ulong LanguageInterfaceImpl::release()
{
    if ( parent )
	return parent->release();
    if ( !--ref ) {
	delete this;
	return 0;
    }
    return ref;
}

QRESULT LanguageInterfaceImpl::queryInterface( const QUuid &uuid, QUnknownInterface** iface )
{
    if ( parent )
	return parent->queryInterface( uuid, iface );

    *iface = 0;
    if ( uuid == IID_QUnknown )
	*iface = (QUnknownInterface*)this;
    else if ( uuid == IID_Language )
	*iface = (LanguageInterface*)this;
    else
	return QE_NOINTERFACE;

    (*iface)->addRef();
    return QS_OK;
}


class NormalizeObject : public QObject
{
public:
    NormalizeObject() : QObject() {}
    static QCString normalizeSignalSlot( const char *signalSlot ) { return QObject::normalizeSignalSlot( signalSlot ); }
};

void LanguageInterfaceImpl::functions( const QString &code, QValueList<Function> *functionMap ) const
{
    QValueList<JavaMethod> l;
    extractJavaMethods( code, &l );
    for ( QValueList<JavaMethod>::Iterator it = l.begin(); it != l.end(); ++it ) {
	Function func;
	func.name = (*it).prototype();
	func.name.remove( 0, (*it).returnType().length() );
	func.body = (*it).body();
	func.returnType = (*it).returnType();
	func.start = (*it).functionStartLineNum();
	func.end = (*it).closingBraceLineNum();
	functionMap->append( func );
    }


/*
	Function	initFunc;
	initFunc.name = QString("init()");
	initFunc.body = QString( "{\n\n}\n" );
	functionMap->append( initFunc );
	
	Function	destroyFunc;
	destroyFunc.name = QString("destroy()");
	destroyFunc.body = QString( "{\n\n}\n" );
	functionMap->append( destroyFunc );
*/
}

QString LanguageInterfaceImpl::createFunctionStart( const QString &className, const QString &method,
						    const QString &returnType,
						    const QString &access )
{
    return access + " " + returnType + " " + method;
}

QStringList LanguageInterfaceImpl::definitions() const
{
    QStringList lst;
//    lst << "Includes (in Implementation)" << "Includes (in Declaration)" << "Forward Declarations" << "Signals";
    return lst;
}

QStringList LanguageInterfaceImpl::definitionEntries( const QString &definition, QUnknownInterface *designerIface ) const
{
    DesignerInterface *iface = 0;
    designerIface->queryInterface( IID_Designer, (QUnknownInterface**) &iface );
    if ( !iface )
	return QStringList();
    DesignerFormWindow *fw = iface->currentForm();
    if ( !fw )
	return QStringList();
    QStringList lst;
/*
    if ( definition == "Includes (in Implementation)" ) {
	lst = fw->implementationIncludes();
    } else if ( definition == "Includes (in Declaration)" ) {
	lst = fw->declarationIncludes();
    } else if ( definition == "Forward Declarations" ) {
	lst = fw->forwardDeclarations();
    } else if ( definition == "Signals" ) {
	lst = fw->signalList();
    }
*/
    iface->release();
    return lst;
}

void LanguageInterfaceImpl::setDefinitionEntries( const QString &definition, const QStringList &entries, QUnknownInterface *designerIface )
{
    DesignerInterface *iface = 0;
    designerIface->queryInterface( IID_Designer, (QUnknownInterface**) &iface );
    if ( !iface )
	return;
    DesignerFormWindow *fw = iface->currentForm();
    if ( !fw )
	return;
/*
    if ( definition == "Includes (in Implementation)" ) {
	fw->setImplementationIncludes( entries );
    } else if ( definition == "Includes (in Declaration)" ) {
	fw->setDeclarationIncludes( entries );
    } else if ( definition == "Forward Declarations" ) {
	fw->setForwardDeclarations( entries );
    } else if ( definition == "Signals" ) {
	fw->setSignalList( entries );
    }
*/
    iface->release();
}

QString LanguageInterfaceImpl::createEmptyFunction()
{
    return "{\n\n}\n";
}

bool LanguageInterfaceImpl::supports( Support s ) const
{
    if ( s == ReturnType )
	return TRUE;
    if ( s == ConnectionsToCustomSlots )
	return TRUE;
    return FALSE;
}

QStringList LanguageInterfaceImpl::fileFilterList() const
{
    QStringList f;
    f << "Java Files (*.java)";
    return f;

}
QStringList LanguageInterfaceImpl::fileExtensionList() const
{
    QStringList f;
    f << "java";
    return f;
}

QString LanguageInterfaceImpl::projectKeyForExtension( const QString &extension ) const
{
 	return "SOURCES";
}

void LanguageInterfaceImpl::sourceProjectKeys( QStringList &keys ) const
{
    keys << "SOURCES";
}

 class CheckObject : public QObject
{
public:
    CheckObject() {}
    bool checkConnectArgs( const char *signal, const char *member ) { return QObject::checkConnectArgs( signal, 0, member ); }

};

bool LanguageInterfaceImpl::canConnect( const QString &signal, const QString &slot )
{
    CheckObject o;
    return o.checkConnectArgs( signal.latin1(), slot.latin1() );
}

void LanguageInterfaceImpl::loadFormCode( const QString &, const QString &filename,
					       QValueList<Function> &functions,
					       QStringList &,
					       QValueList<Connection> & )
{
    QFile f( filename );
    if ( !f.open( IO_ReadOnly ) )
	return;
    QTextStream ts( &f );
    QString code( ts.read() );
    this->functions( code, &functions );
}

// This method really belongs here, as it's language specific. The C++ version is in
// designer/designer/formfile.cpp, but really belongs in the cppeditor plugin.
// But there's no way to call it at the appropriate time to insert it in the 
// start of the <form name>.ui.java file 
QString LanguageInterfaceImpl::codeComment() const
{
    return QString( "/****************************************************************************\n"
    "** ui.java extension file, included from the uic-generated form implementation.\n"
    "**\n"
    "** If you wish to add, delete or rename functions or slots use\n"
    "** Qt Designer which will update this file, preserving your code. Create an\n"
    "** init() method in place of a constructor, and a destroy() method in\n"
    "** place of a finalizer.\n"
    "*****************************************************************************/\n" );
}

void LanguageInterfaceImpl::preferedExtensions( QMap<QString, QString> &extensionMap ) const
{
    extensionMap.insert( "java", "Java Source File" );
}

QStrList LanguageInterfaceImpl::signalNames( QObject *obj ) const
{
    QStrList sigs;
    QStrList javasigs;
    sigs = obj->metaObject()->signalNames( TRUE );
    sigs.remove( "destroyed()" );
    
	for ( QPtrListIterator<char> sit( sigs ); sit.current(); ++sit ) {
	QString f( sit.current() );
	
	// It looks as though the purpose of this method is to allow the C++
	// types in the signals to be changed to another language like java
	// by editing the strings as below. However, unfortunately it doesn't 
	// have any effect :-(
	f.replace("unsigned char", "byte");
	f.replace("bool", "boolean");
	f.replace("QString", "String");
	f.replace("QTime", "Date");
	f.replace("::", ".");
	f.remove("*");
	f.remove("&");
	f.remove("const ");
	f.remove("signed ");
	f.remove("unsigned ");
	javasigs.append(strdup((const char *) f));
    }
 
    return javasigs;
}
