/**********************************************************************
**
** Copyright (C) 2000 Trolltech AS.  All rights reserved.
**
** This file is part of Qt Designer.
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.trolltech.com/gpl/ for GPL licensing information.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
**********************************************************************/

#include "sourcetemplateinterfaceimpl.h"
#include <designerinterface.h>
#include "mainfilesettings.h"
#include <qlineedit.h>
#include <qlistbox.h>

SourceTemplateInterfaceImpl::SourceTemplateInterfaceImpl()
{
}

QRESULT SourceTemplateInterfaceImpl::queryInterface( const QUuid &uuid, QUnknownInterface **iface )
{
    *iface = 0;
    if ( uuid == IID_QUnknown )
	*iface = (QUnknownInterface*)this;
    else if ( uuid == IID_QFeatureList )
	*iface = (QFeatureListInterface*)this;
    else if ( uuid == IID_SourceTemplate )
	*iface = (SourceTemplateInterface*)this;
    else
	return QE_NOINTERFACE;

    (*iface)->addRef();
    return QS_OK;
}

QStringList SourceTemplateInterfaceImpl::featureList() const
{
    QStringList l;
    l << "QtJava Main-File (Main.java)";
    return l;
}

static QString generateMainJavaCode( const QString &formname, const QString &mainClass )
{
    QString code;
    code += "import org.kde.qt.*;\n";
    code += "import " + formname + ";\n\n";
    code += "class " + mainClass + " extends Qt {\n\n";
    code += "    public static void main( String[] args )\n";
    code += "    {\n";
    code += "        QApplication a = new QApplication( args );\n";
    code += "        " + formname + " w = new " + formname + "();\n";
    code += "        w.show();\n";
    code += "        a.connect( a, SIGNAL( \"lastWindowClosed()\" ), a, SLOT( \"quit()\" ) );\n";
    code += "        a.exec();\n";
    code += "        return;\n";
    code += "    }\n\n";
    code += "    static {\n";
    code += "    	qtjava.initialize();\n";
    code += "    }\n";
    code += "}\n";
    return code;
}

SourceTemplateInterface::Source SourceTemplateInterfaceImpl::create( const QString &templ,
								     QUnknownInterface *appIface )
{
    SourceTemplateInterface::Source src;
    src.type = SourceTemplateInterface::Source::Invalid;
    if ( templ == "QtJava Main-File (Main.java)" ) {
	JavaMainFile dia( 0, 0, TRUE );
	dia.setup( appIface );
	if ( dia.exec() == QDialog::Accepted ) {
	    DesignerInterface *dIface = 0;
	    appIface->queryInterface( IID_Designer, (QUnknownInterface**)&dIface );
	    if ( dIface ) {
		src.type = SourceTemplateInterface::Source::FileName;
		src.filename = dia.editFileName->text();
		QString mainClass = src.filename;
//		Remove the .java suffix from the source filename
		mainClass.remove( mainClass.length() - 5, 5 );
		src.code = generateMainJavaCode( dia.listForms->text( dia.listForms->currentItem() ),
						mainClass );
	    }
	}
    }
    return src;
}

QString SourceTemplateInterfaceImpl::language( const QString & ) const
{
    return "Java";
}
